@@ -1,7 +1,8 @@
 import boto3
-from kvirt.common import success, info2, pprint, error, fix_typos
+from kvirt.common import success, info2, pprint, error, fix_typos, warning
 import os
 import yaml
+from yaml import safe_dump, safe_load
 
 supported_versions = ['1.20', '1.21', '1.22', '1.23', '1.24', '1.25', '1.26', '1.27']
 
@@ -45,13 +46,13 @@ def get_kubeconfig(config, cluster, zonal=True):
                       "users": [{"name": cluster_arn, "user": {"exec": {
                           "apiVersion": "client.authentication.k8s.io/v1beta1", "command": "ekstoken",
                           "interactiveMode": "Never", "args": [config.client, cluster]}}}]}
-    config_text = yaml.dump(cluster_config, default_flow_style=False)
+    config_text = safe_dump(cluster_config, default_flow_style=False)
     clusterdir = os.path.expanduser(f"~/.kcli/clusters/{cluster}")
     with open(f"{clusterdir}/auth/kubeconfig", 'w') as f:
         f.write(config_text)
 
 
-def scale(config, cluster, overrides):
+def scale(config, plandir, cluster, overrides):
     data = {'workers': 2,
             'network': 'default',
             'role': None,
@@ -90,18 +91,8 @@ def scale(config, cluster, overrides):
     return {'result': 'success'}
 
 
-def create(config, cluster, overrides, dnsconfig=None):
-    data = {'workers': 2,
-            'network': 'default',
-            'extra_networks': [],
-            'ctlplane_role': None,
-            'worker_role': None,
-            'security_group': None,
-            'disk_size': None,
-            'flavor': None,
-            'ami_type': None,
-            'capacity_type': None,
-            'version': None}
+def create(config, plandir, cluster, overrides, dnsconfig=None):
+    data = safe_load(open(f'{plandir}/kcli_default.yml'))
     data.update(overrides)
     fix_typos(data)
     k = config.k
@@ -113,14 +104,29 @@ def create(config, cluster, overrides, dnsconfig=None):
     flavor = data['flavor']
     ami_type = data['ami_type']
     capacity_type = data['capacity_type']
-    network = data['network']
-    extra_networks = data['extra_networks']
-    if not extra_networks:
-        return {'result': 'failure', 'reason': 'You must define extra_networks'}
+    network = data['subnet'] or data['network']
+    extra_networks = data['extra_subnets'] or data['extra_networks']
     sgid = data['security_group']
     plan = cluster
     tags = {'plan': cluster, 'kube': cluster, 'kubetype': 'eks'}
     cluster_data = {'name': cluster, 'tags': tags}
+    auto_mode = data['auto_mode']
+    if not data['default_addons']:
+        warning("Disabling network add-ons (and automode)")
+        cluster_data['bootstrapSelfManagedAddons'] = True
+        auto_mode = False
+    extended_support = data['extended_support']
+    if not extended_support:
+        cluster_data['upgradePolicy'] = {'supportType': 'STANDARD'}
+    zonal_shift = data['zonal_shift']
+    if zonal_shift:
+        cluster_data['zonalShiftConfig'] = {'enabled': True}
+    logging = data['logging']
+    if logging:
+        logging_data = []
+        for _type in data['logging_types']:
+            logging_data.append({'type': _type, 'enabled': True})
+        cluster_data['logging'] = {'clusterLogging': logging_data}
     if version is not None:
         version = str(version)
         if version not in supported_versions:
@@ -141,12 +147,13 @@ def create(config, cluster, overrides, dnsconfig=None):
             installparam['client'] = config.client
             yaml.safe_dump(installparam, p, default_flow_style=False, encoding='utf-8', allow_unicode=True)
     access_key_id, access_key_secret, session_token, region = project_init(config)
-    ctlplane_roles = list_valid_roles(config, 'AmazonEKSClusterPolicy')
+    cluster_role = 'AmazonEC2FullAccess' if auto_mode else 'AmazonEKSClusterPolicy'
+    ctlplane_roles = list_valid_roles(config, cluster_role)
     if ctlplane_role is not None:
         if ctlplane_role not in ctlplane_roles:
             return {'result': 'failure', 'reason': f"Invalid role {ctlplane_role}"}
     elif not ctlplane_roles:
-        return {'result': 'failure', 'reason': "No role with AmazonEKSClusterPolicy found"}
+        return {'result': 'failure', 'reason': f"No role with {cluster_role} found"}
     else:
         ctlplane_role = [*ctlplane_roles][0]
         pprint(f"Using ctlplane role {ctlplane_role}")
@@ -163,14 +170,35 @@ def create(config, cluster, overrides, dnsconfig=None):
         pprint(f"Using worker role {worker_role}")
     worker_role = worker_roles[worker_role]
     subnetids = []
-    for index, n in enumerate([network] + extra_networks):
-        vpcid, subnetid = k.eks_get_network(n)
+    total_subnets = [network] + extra_networks
+    for index, n in enumerate(total_subnets):
+        vpcid, subnetid, az = k.eks_get_network(n)
         if index == 0:
             sgid = k.get_security_group_id(n, vpcid) if sgid is not None else k.get_default_security_group_id(vpcid)
             if sgid is None:
                 return {'result': 'failure', 'reason': "Couldn't find a valid security group"}
         subnetids.append(subnetid)
+    if len(total_subnets) == 1:
+        subnets = k.list_subnets()
+        subnetid = None
+        for subnetname in subnets:
+            subnet = subnets[subnetname]
+            if subnet['network'] == vpcid and subnet['id'] != subnetid and subnet['az'] != az:
+                subnetid = subnet['id']
+                break
+        if subnetid is None:
+            return {'result': 'failure', 'reason': "Couldn't find a valid subnet in the same vpc but with other az"}
+        else:
+            pprint(f"Using subnet {subnetid} as extra subnet")
+            subnetids.append(subnetid)
     cluster_data['resourcesVpcConfig'] = {'subnetIds': subnetids, 'securityGroupIds': [sgid]}
+    if auto_mode:
+        auto_mode_dict = {'storageConfig': {'blockStorage': {'enabled': True}},
+                          'kubernetesNetworkConfig': {'elasticLoadBalancing': {'enabled': True}},
+                          'computeConfig': {'enabled': True, 'nodePools': ['general-purpose', 'system'],
+                                            'nodeRoleArn': worker_role},
+                          'accessConfig': {'authenticationMode': 'API_AND_CONFIG_MAP'}}
+        cluster_data.update(auto_mode_dict)
     eks = boto3.client('eks', aws_access_key_id=access_key_id, aws_secret_access_key=access_key_secret,
                        region_name=region, aws_session_token=session_token)
     pprint(f"Creating cluster {cluster}")
@@ -181,28 +209,29 @@ def create(config, cluster, overrides, dnsconfig=None):
     waiter = eks.get_waiter("cluster_active")
     waiter.wait(name=cluster)
     get_kubeconfig(config, cluster)
-    nodegroup_data = {'clusterName': cluster, 'nodegroupName': cluster, 'scalingConfig':
-                      {'minSize': workers, 'maxSize': 50, 'desiredSize': workers}, 'subnets': subnetids, 'tags': tags,
-                      'nodeRole': worker_role}
-    keypair = config.options.get('keypair')
-    if keypair is not None:
-        nodegroup_data['remoteAccess'] = {'ec2SshKey': keypair, 'sourceSecurityGroups': [sgid]}
-    if version is not None:
-        nodegroup_data['version'] = version
-    if disk_size is not None:
-        nodegroup_data['diskSize'] = disk_size
-    if flavor is not None:
-        nodegroup_data['instanceTypes'] = [flavor]
-    if ami_type is not None:
-        nodegroup_data['amiType'] = ami_type
-    if capacity_type is not None:
-        nodegroup_data['capacityType'] = capacity_type
-    pprint(f"Creating nodegroup {cluster}")
-    response = eks.create_nodegroup(**nodegroup_data)
-    if config.debug:
-        print(response)
-    waiter = eks.get_waiter("cluster_active")
-    waiter.wait(name=cluster)
+    if not auto_mode:
+        nodegroup_data = {'clusterName': cluster, 'nodegroupName': cluster, 'scalingConfig':
+                          {'minSize': workers, 'maxSize': 50, 'desiredSize': workers}, 'subnets': subnetids,
+                          'tags': tags, 'nodeRole': worker_role}
+        keypair = config.options.get('keypair')
+        if keypair is not None:
+            nodegroup_data['remoteAccess'] = {'ec2SshKey': keypair, 'sourceSecurityGroups': [sgid]}
+        if version is not None:
+            nodegroup_data['version'] = version
+        if disk_size is not None:
+            nodegroup_data['diskSize'] = disk_size
+        if flavor is not None:
+            nodegroup_data['instanceTypes'] = [flavor]
+        if ami_type is not None:
+            nodegroup_data['amiType'] = ami_type
+        if capacity_type is not None:
+            nodegroup_data['capacityType'] = capacity_type
+        pprint(f"Creating nodegroup {cluster}")
+        response = eks.create_nodegroup(**nodegroup_data)
+        if config.debug:
+            print(response)
+        waiter = eks.get_waiter("cluster_active")
+        waiter.wait(name=cluster)
     success(f"Kubernetes cluster {cluster} deployed!!!")
     info2(f"export KUBECONFIG=$HOME/.kcli/clusters/{cluster}/auth/kubeconfig")
     info2("export PATH=$PWD:$PATH")
