@@ -1,8 +1,8 @@
 from base64 import b64encode
 from glob import glob
 from ipaddress import ip_network
-from kvirt.common import success, error, pprint, info2, container_mode, warning, fix_typos
-from kvirt.common import get_oc, pwd_path, get_installer_rhcos, get_ssh_pub_key, start_baremetal_hosts, olm_app
+from kvirt.common import success, error, pprint, info2, container_mode, warning, fix_typos, olm_app
+from kvirt.common import get_oc, pwd_path, get_installer_rhcos, get_ssh_pub_key, start_baremetal_hosts_with_iso
 from kvirt.common import deploy_cloud_storage
 from kvirt.cluster.openshift import get_ci_installer, get_downstream_installer, get_installer_version
 from kvirt.cluster.openshift import same_release_images, process_apps, offline_image
@@ -20,7 +20,7 @@ from urllib.parse import urlparse
 from urllib.request import urlopen, Request
 from yaml import safe_dump, safe_load
 
-virt_providers = ['kvm', 'kubevirt', 'ovirt', 'openstack', 'vsphere', 'proxmox']
+virt_providers = ['kvm', 'kubevirt', 'ovirt', 'openstack', 'proxmox', 'vsphere']
 cloud_providers = ['aws', 'azure', 'gcp', 'ibm', 'hcloud']
 
 
@@ -226,20 +226,25 @@ def scale(config, plandir, cluster, overrides):
         overrides['cluster'] = cluster
         overrides['clusterdir'] = clusterdir
         plan = overrides.get('plan') or plan
-        if 'ingress_ip' not in overrides and provider != 'kubevirt':
-            msg = "Missing ingress_ip..."
-            return {'result': 'failure', 'reason': msg}
-        domain = overrides.get('domain')
-        if domain is None:
-            msg = "Missing domain..."
-            return {'result': 'failure', 'reason': msg}
-        if 'management_ingress_domain' not in overrides:
-            overrides['management_ingress_domain'] = f'apps.{cluster}.{domain}'
-        os.mkdir(clusterdir)
-        ignitionscript = config.process_inputfile(cluster, f"{plandir}/ignition.sh", overrides=overrides)
-        with open(f"{clusterdir}/ignition.sh", 'w') as f:
-            f.write(ignitionscript)
-        call(f'bash {clusterdir}/ignition.sh', shell=True)
+        namespace = data['namespace']
+        assisted = cluster in os.popen(f'oc get -n {namespace}-{cluster} infraenv {cluster}').read().strip()
+        if not assisted:
+            if 'ingress_ip' not in overrides and provider != 'kubevirt':
+                msg = "Missing ingress_ip..."
+                return {'result': 'failure', 'reason': msg}
+            domain = overrides.get('domain')
+            if domain is None:
+                msg = "Missing domain..."
+                return {'result': 'failure', 'reason': msg}
+            if 'management_ingress_domain' not in overrides:
+                overrides['management_ingress_domain'] = f'apps.{cluster}.{domain}'
+            ignitionscript = config.process_inputfile(cluster, f"{plandir}/ignition.sh", overrides=overrides)
+            with open(f"{clusterdir}/ignition.sh", 'w') as f:
+                f.write(ignitionscript)
+            call(f'bash {clusterdir}/ignition.sh', shell=True)
+        else:
+            os.mkdir(clusterdir)
+            data['assisted'] = True
     old_extra_nodepools = []
     old_assisted_vms_number = 2
     if storedparameters and os.path.exists(f"{clusterdir}/kcli_parameters.yml"):
@@ -310,12 +315,10 @@ def scale(config, plandir, cluster, overrides):
         if assisted:
             all_baremetal_hosts = old_baremetal_hosts + baremetal_hosts
             create_bmh_objects(config, plandir, cluster, namespace, all_baremetal_hosts, overrides)
-            total = f'oc get nodepool -n {namespace} -o jsonpath='
-            total += '"{range .items[?(@.spec.clusterName==\\"%s\\")]}{.spec.replicas}{\'\\n\'}{end}"' % cluster
-            total += " | awk '{sum += $1} END {print sum}'"
-            total_replicas = int(os.popen(total).read())
-            number = overrides.get('workers', 2) - total_replicas if nodepool != cluster else len(all_baremetal_hosts)
-            cmcmd = f"oc -n {data['namespace']} scale nodepool {nodepool} --replicas {number}"
+            replicas_cmd = "oc get nodepool -n %s %s -o jsonpath='{.spec.replicas}'" % (namespace, nodepool)
+            replicas = int(os.popen(replicas_cmd).read()) + len(baremetal_hosts)
+            pprint(f"Setting replicas to {replicas} in nodepool {nodepool}")
+            cmcmd = f"oc -n {data['namespace']} scale nodepool {nodepool} --replicas {replicas}"
             call(cmcmd, shell=True)
             return {'result': 'success'}
         else:
@@ -327,7 +330,9 @@ def scale(config, plandir, cluster, overrides):
                 svcport_cmd = 'oc get svc -n default httpd-kcli-svc -o yaml'
                 svcport = safe_load(os.popen(svcport_cmd).read())['spec']['ports'][0]['nodePort']
                 iso_url = f'http://{svcip}:{svcport}/{cluster}-worker.iso'
-            start_baremetal_hosts(baremetal_hosts, iso_url, overrides=overrides, debug=config.debug)
+            result = start_baremetal_hosts_with_iso(baremetal_hosts, iso_url, overrides=overrides, debug=config.debug)
+            if result['result'] != 'success':
+                return result
             worker_overrides['workers'] = workers - len(new_baremetal_hosts)
     if platform is not None:
         return {'result': 'success'}
@@ -349,24 +354,17 @@ def create(config, plandir, cluster, overrides):
     elif not os.path.isabs(os.environ['KUBECONFIG']):
         os.environ['KUBECONFIG'] = f"{os.getcwd()}/{os.environ['KUBECONFIG']}"
         if not os.path.exists(os.environ['KUBECONFIG']):
-            msg = "Kubeconfig not found. Leaving..."
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': "Kubeconfig not found. Leaving..."}
     data = safe_load(open(f'{plandir}/kcli_default.yml'))
     data.update(overrides)
     fix_typos(data)
     retries = data.get('retries')
-    if 'cluster' in overrides:
-        clustervalue = overrides.get('cluster')
-    elif cluster is not None:
-        clustervalue = cluster
-    else:
-        clustervalue = 'myhypershift'
-    data['cluster'] = clustervalue
+    data['cluster'] = cluster
     data['kube'] = data['cluster']
     data['kubetype'] = 'hypershift'
     ignore_hosts = data['ignore_hosts'] or data['sslip']
-    pprint(f"Deploying cluster {clustervalue}")
-    plan = cluster if cluster is not None else clustervalue
+    pprint(f"Deploying cluster {cluster}")
+    plan = cluster
     upstream = data['upstream']
     platform = data.get('platform')
     assisted = platform == 'assisted'
@@ -375,8 +373,7 @@ def create(config, plandir, cluster, overrides):
     kubevirt = platform == 'kubevirt'
     data['kubevirt'] = kubevirt
     if platform not in [None, 'kubevirt', 'assisted']:
-        msg = "Incorrect platform. Choose between None, kubevirt and assisted"
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': "Incorrect platform. Choose between None, kubevirt and assisted"}
     baremetal_iso = data.get('baremetal_iso', False)
     baremetal_hosts = data.get('baremetal_hosts')
     async_install = data.get('async')
@@ -391,8 +388,7 @@ def create(config, plandir, cluster, overrides):
     use_management_version = False
     version = data.get('version')
     if version not in ['dev-preview', 'stable', 'ci', 'nightly', 'latest', 'cluster']:
-        msg = f"Invalid version {version}"
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': f"Invalid version {version}"}
     elif version == 'cluster':
         use_management_version = True
         version = 'stable'
@@ -403,8 +399,7 @@ def create(config, plandir, cluster, overrides):
         data['tag'] = tag
     pull_secret = os.path.expanduser(pwd_path(data.get('pull_secret')))
     if not os.path.exists(pull_secret):
-        msg = f"Missing pull secret file {pull_secret}"
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': f"Missing pull secret file {pull_secret}"}
     network_type = data['network_type']
     if network_type not in ['Calico', 'OVNKubernetes']:
         data['network_type'] = 'Other'
@@ -412,8 +407,7 @@ def create(config, plandir, cluster, overrides):
     namespace = data.get('namespace')
     clusterdir = os.path.expanduser(f"~/.kcli/clusters/{cluster}")
     if os.path.exists(clusterdir):
-        msg = f"Remove existing directory {clusterdir} or use --force"
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': f"Remove existing directory {clusterdir} or use --force"}
     else:
         os.makedirs(f"{clusterdir}/auth")
     if which('oc') is None:
@@ -426,14 +420,15 @@ def create(config, plandir, cluster, overrides):
             pprint(f"Using default class {sc['metadata']['name']}")
             default_sc = True
     if not default_sc:
-        msg = "Default Storage class not found. Leaving..."
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': "Default Storage class not found. Leaving..."}
     kubevirt_crd_cmd = 'oc get crd hyperconvergeds.hco.kubevirt.io -o yaml 2>/dev/null'
     if kubevirt and safe_load(os.popen(kubevirt_crd_cmd).read()) is None:
         warning("Kubevirt not fully installed. Installing it for you")
         app_name, source, channel, csv, description, x_namespace, channels, crds = olm_app('kubevirt-hyperconverged')
         app_data = {'name': app_name, 'source': source, 'channel': channel, 'namespace': x_namespace, 'csv': csv}
-        config.create_app_openshift(app_name, app_data)
+        result = config.create_app_openshift(app_name, app_data)
+        if result != 0:
+            return {'result': 'failure', 'reason': "Couldnt install kubevirt properly"}
     kubeconfig = os.environ.get('KUBECONFIG')
     kubeconfigdir = os.path.dirname(kubeconfig) if kubeconfig is not None else os.path.expanduser("~/.kube")
     kubeconfig = os.path.basename(kubeconfig) if kubeconfig is not None else 'config'
@@ -452,16 +447,16 @@ def create(config, plandir, cluster, overrides):
         app_data = {'name': app_name, 'source': source, 'channel': channel, 'namespace': x_namespace, 'csv': csv,
                     'mce_hypershift': mce_hypershift, 'assisted': need_assisted, 'version': version, 'tag': tag,
                     'pull_secret': pull_secret}
-        config.create_app_openshift(app_name, app_data)
+        result = config.create_app_openshift(app_name, app_data)
+        if result != 0:
+            return {'result': 'failure', 'reason': "Couldnt install mce properly"}
         sleep(240)
         if assisted and safe_load(os.popen(assisted_crd_cmd).read()) is None:
-            msg = "Couldnt install assisted via mce properly"
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': "Couldnt install assisted via mce properly"}
     if need_hypershift and upstream:
         warning("Hypershift needed. Installing it for you")
         if which('podman') is None:
-            msg = "Please install podman first in order to install hypershift"
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': "Please install podman first in order to install hypershift"}
         else:
             hypercmd = f"podman pull {data['operator_image']}"
             call(hypercmd, shell=True)
@@ -471,8 +466,7 @@ def create(config, plandir, cluster, overrides):
             call(hypercmd, shell=True)
             sleep(120)
     if safe_load(os.popen(hosted_crd_cmd).read()) is None:
-        msg = "Couldnt install hypershift properly"
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': "Couldnt install hypershift properly"}
     registry = 'quay.io'
     management_image = os.popen("oc get clusterversion version -o jsonpath='{.status.desired.image}'").read()
     prefixes = ['quay.io', 'registry.ci']
@@ -514,8 +508,7 @@ def create(config, plandir, cluster, overrides):
         if version == 'latest':
             tag = versions[0]
         elif str(tag) not in versions:
-            msg = f"Invalid tag {tag}. Choose between {','.join(versions)}"
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': f"Invalid tag {tag}. Choose between {','.join(versions)}"}
     else:
         warning(f"Couldnt verify whether {tag} is a valid tag")
     management_cmd = "oc get ingresscontroller -n openshift-ingress-operator default -o jsonpath='{.status.domain}'"
@@ -559,8 +552,7 @@ def create(config, plandir, cluster, overrides):
     elif os.path.exists(pub_key):
         data['pub_key'] = open(pub_key).read().strip()
     else:
-        msg = f"Publickey file {pub_key} not found"
-        return {'result': 'failure', 'reason': msg}
+        return {'result': 'failure', 'reason': f"Public key file {pub_key} not found"}
     network = data.get('network')
     ingress_ip = data.get('ingress_ip')
     virtual_router_id = None
@@ -590,8 +582,7 @@ def create(config, plandir, cluster, overrides):
                     route_cmd += f"--hostname={hostname} --wildcard-policy=Subdomain --port=443"
                     call(route_cmd, shell=True)
                 elif ingress_ip is None:
-                    msg = f"Couldnt gather an ingress_ip from network {network}"
-                    return {'result': 'failure', 'reason': msg}
+                    return {'result': 'failure', 'reason': f"Couldnt gather an ingress_ip from network {network}"}
         elif provider == 'kvm' and networkinfo['type'] == 'routed':
             cidr = networkinfo['cidr']
             ingress_index = 3 if ':' in cidr else -4
@@ -599,8 +590,7 @@ def create(config, plandir, cluster, overrides):
             warning(f"Using {ingress_ip} as ingress_ip")
             data['ingress_ip'] = ingress_ip
         else:
-            msg = "You need to define ingress_ip in your parameters file"
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': "You need to define ingress_ip in your parameters file"}
     elif kubevirt:
         warning(f"Note that ingress_ip {ingress_ip} won't be configured as you're using kubevirt provider")
     if ingress_ip is not None and data.get('virtual_router_id') is None and not kubevirt_ingress_svc:
@@ -638,6 +628,7 @@ def create(config, plandir, cluster, overrides):
                 pprint(f"Injecting manifest {f}")
                 mc_data = json.dumps(mc_data)
                 manifests.append({'name': mc_name, 'data': mc_data})
+    ingress_metallb = data['assisted_ingress_metallb']
     if assisted:
         keepalived_yml_data = config.process_inputfile(cluster, f"{plandir}/staticpods/keepalived.yml", overrides=data)
         keepalived_yml_data = b64encode(keepalived_yml_data.encode()).decode("UTF-8")
@@ -657,9 +648,11 @@ def create(config, plandir, cluster, overrides):
         assisted_data['cluster'] = cluster
         assisted_data['sslip'] = sslip
         assisted_data['coredns'] = coredns
-        assisted_data = config.process_inputfile(cluster, f'{plandir}/assisted_ingress.yml', overrides=assisted_data)
-        assisted_data = json.dumps(assisted_data)
-        manifests.append({'name': f'assisted-ingress-{cluster}', 'data': assisted_data})
+        if not ingress_metallb:
+            assisted_data = config.process_inputfile(cluster, f'{plandir}/assisted_ingress.yml',
+                                                     overrides=assisted_data)
+            assisted_data = json.dumps(assisted_data)
+            manifests.append({'name': f'assisted-ingress-{cluster}', 'data': assisted_data})
     async_files = []
     async_tempdir = TemporaryDirectory()
     asyncdir = async_tempdir.name
@@ -756,8 +749,7 @@ def create(config, plandir, cluster, overrides):
         elif version in ['dev-preview', 'stable', 'ci', 'nightly', 'latest']:
             assetsdata['hosted_image'] = offline_image(version=version, tag=tag, pull_secret=pull_secret)
         else:
-            msg = f"Invalid version {version}"
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': f"Invalid version {version}"}
     hostedclusterfile = config.process_inputfile(cluster, f"{plandir}/hostedcluster.yaml", overrides=assetsdata)
     with open(f"{clusterdir}/hostedcluster.yaml", 'w') as f:
         f.write(hostedclusterfile)
@@ -772,8 +764,7 @@ def create(config, plandir, cluster, overrides):
         elif version in ['dev-preview', 'stable', 'latest']:
             run = get_downstream_installer(version=version, tag=tag, pull_secret=pull_secret)
         else:
-            msg = f"Invalid version {version}"
-            return {'result': 'failure', 'reason': msg}
+            return {'result': 'failure', 'reason': f"Invalid version {version}"}
         if run != 0:
             msg = "Couldn't download openshift-install"
             return {'result': 'failure', 'reason': msg}
@@ -897,7 +888,9 @@ def create(config, plandir, cluster, overrides):
                 f.write(json.dumps(new_ignition))
         elif baremetal_iso or baremetal_hosts:
             iso_url = handle_baremetal_iso(config, plandir, cluster, data, baremetal_hosts)
-            start_baremetal_hosts(baremetal_hosts, iso_url, overrides=overrides, debug=config.debug)
+            result = start_baremetal_hosts_with_iso(baremetal_hosts, iso_url, overrides=overrides, debug=config.debug)
+            if result['result'] != 'success':
+                return result
             data['workers'] = data.get('workers', 2) - len(baremetal_hosts)
     pprint("Waiting for kubeconfig to be available")
     call(f"until oc -n {namespace} get secret {cluster}-admin-kubeconfig >/dev/null 2>&1 ; do sleep 1 ; done",
@@ -965,6 +958,16 @@ def create(config, plandir, cluster, overrides):
     apps = overrides.get('apps', [])
     overrides['hypershift'] = True
     overrides['cluster'] = cluster
+    if ingress_metallb:
+        if 'metallb-operator' not in apps:
+            apps.append('metallb-operator')
+        if 'metallb_ranges' not in overrides:
+            overrides['metallb_ranges'] = [ingress_ip]
+        ingress_file = config.process_inputfile('xxx', f'{plandir}/assisted_ingress_service.yml.j2',
+                                                overrides={'ingress_ip': ingress_ip})
+        with open(f"{clusterdir}/assisted_ingress_service.yml", 'w') as f:
+            f.write(ingress_file)
+        call(f'oc create -f {clusterdir}/assisted_ingress_service.yml', shell=True)
     process_apps(config, clusterdir, apps, overrides)
     if autoscale:
         config.import_in_kube(network=network, secure=True)
